"""This file defines MIDI standard constants, which are useful for converting
between numeric MIDI data values and human-readable text.

"""

# INSTRUMENT_MAP[program_number] maps the program_number to an instrument name
INSTRUMENT_MAP = ['Acoustic Grand Piano', 'Bright Acoustic Piano',
                  'Electric Grand Piano', 'Honky-tonk Piano',
                  'Electric Piano 1', 'Electric Piano 2', 'Harpsichord',
                  'Clavinet', 'Celesta', 'Glockenspiel', 'Music Box',
                  'Vibraphone', 'Marimba', 'Xylophone', 'Tubular Bells',
                  'Dulcimer', 'Drawbar Organ', 'Percussive Organ',
                  'Rock Organ', 'Church Organ', 'Reed Organ', 'Accordion',
                  'Harmonica', 'Tango Accordion', 'Acoustic Guitar (nylon)',
                  'Acoustic Guitar (steel)', 'Electric Guitar (jazz)',
                  'Electric Guitar (clean)', 'Electric Guitar (muted)',
                  'Overdriven Guitar', 'Distortion Guitar',
                  'Guitar Harmonics', 'Acoustic Bass',
                  'Electric Bass (finger)', 'Electric Bass (pick)',
                  'Fretless Bass', 'Slap Bass 1', 'Slap Bass 2',
                  'Synth Bass 1', 'Synth Bass 2', 'Violin', 'Viola', 'Cello',
                  'Contrabass', 'Tremolo Strings', 'Pizzicato Strings',
                  'Orchestral Harp', 'Timpani', 'String Ensemble 1',
                  'String Ensemble 2', 'Synth Strings 1', 'Synth Strings 2',
                  'Choir Aahs', 'Voice Oohs', 'Synth Choir', 'Orchestra Hit',
                  'Trumpet', 'Trombone', 'Tuba', 'Muted Trumpet',
                  'French Horn', 'Brass Section', 'Synth Brass 1',
                  'Synth Brass 2', 'Soprano Sax', 'Alto Sax', 'Tenor Sax',
                  'Baritone Sax', 'Oboe', 'English Horn', 'Bassoon',
                  'Clarinet', 'Piccolo', 'Flute', 'Recorder', 'Pan Flute',
                  'Blown bottle', 'Shakuhachi', 'Whistle', 'Ocarina',
                  'Lead 1 (square)', 'Lead 2 (sawtooth)',
                  'Lead 3 (calliope)', 'Lead 4 chiff', 'Lead 5 (charang)',
                  'Lead 6 (voice)', 'Lead 7 (fifths)',
                  'Lead 8 (bass + lead)', 'Pad 1 (new age)', 'Pad 2 (warm)',
                  'Pad 3 (polysynth)', 'Pad 4 (choir)', 'Pad 5 (bowed)',
                  'Pad 6 (metallic)', 'Pad 7 (halo)', 'Pad 8 (sweep)',
                  'FX 1 (rain)', 'FX 2 (soundtrack)', 'FX 3 (crystal)',
                  'FX 4 (atmosphere)', 'FX 5 (brightness)', 'FX 6 (goblins)',
                  'FX 7 (echoes)', 'FX 8 (sci-fi)', 'Sitar', 'Banjo',
                  'Shamisen', 'Koto', 'Kalimba', 'Bagpipe', 'Fiddle',
                  'Shanai', 'Tinkle Bell', 'Agogo', 'Steel Drums',
                  'Woodblock', 'Taiko Drum', 'Melodic Tom', 'Synth Drum',
                  'Reverse Cymbal', 'Guitar Fret Noise', 'Breath Noise',
                  'Seashore', 'Bird Tweet', 'Telephone Ring', 'Helicopter',
                  'Applause', 'Gunshot']

# INSTRUMENT_CLASSES contains the classes present in INSTRUMENTS
INSTRUMENT_CLASSES = ['Piano', 'Chromatic Percussion', 'Organ', 'Guitar',
                      'Bass', 'Strings', 'Ensemble', 'Brass', 'Reed', 'Pipe',
                      'Synth Lead', 'Synth Pad', 'Synth Effects', 'Ethnic',
                      'Percussive',
                      'Sound Effects']

# List which maps MIDI note number - 35 to drum name
# from http://www.midi.org/techspecs/gm1sound.php
DRUM_MAP = ['Acoustic Bass Drum', 'Bass Drum 1', 'Side Stick',
            'Acoustic Snare', 'Hand Clap', 'Electric Snare',
            'Low Floor Tom', 'Closed Hi Hat', 'High Floor Tom',
            'Pedal Hi Hat', 'Low Tom', 'Open Hi Hat',
            'Low-Mid Tom', 'Hi-Mid Tom', 'Crash Cymbal 1',
            'High Tom', 'Ride Cymbal 1', 'Chinese Cymbal',
            'Ride Bell', 'Tambourine', 'Splash Cymbal',
            'Cowbell', 'Crash Cymbal 2', 'Vibraslap',
            'Ride Cymbal 2', 'Hi Bongo', 'Low Bongo',
            'Mute Hi Conga', 'Open Hi Conga', 'Low Conga',
            'High Timbale', 'Low Timbale', 'High Agogo',
            'Low Agogo', 'Cabasa', 'Maracas',
            'Short Whistle', 'Long Whistle', 'Short Guiro',
            'Long Guiro', 'Claves', 'Hi Wood Block',
            'Low Wood Block', 'Mute Cuica', 'Open Cuica',
            'Mute Triangle', 'Open Triangle']


# from https://www.midi.org/specifications-old/item/table-3-control-change-messages-data-bytes-2
CONTROL_CHANGE_MAP = {
    0: 'Bank Select',
    1: 'Modulation Wheel or Lever',
    2: 'Breath Controller',
    3: 'Undefined',
    4: 'Foot Controller',
    5: 'Portamento Time',
    6: 'Data Entry MSB',
    7: 'Channel Volume (formerly Main Volume)',
    8: 'Balance',
    9: 'Undefined',
    10: 'Pan',
    11: 'xpression Controller',
    12: 'Effect Control',
    13: 'Effect Control',
    14: 'Undefined',
    15: 'Undefined',
    16: 'General Purpose Controller',
    17: 'General Purpose Controller',
    18: 'General Purpose Controller',
    19: 'General Purpose Controller',
    20: 'Undefined',
    21: 'Undefined',
    22: 'Undefined',
    23: 'Undefined',
    24: 'Undefined',
    25: 'Undefined',
    26: 'Undefined',
    27: 'Undefined',
    28: 'Undefined',
    29: 'Undefined',
    30: 'Undefined',
    31: 'Undefined',
    32: 'LSB for Control 0',
    33: 'LSB for Control 1',
    34: 'LSB for Control 2',
    35: 'LSB for Control 3',
    36: 'LSB for Control 4',
    37: 'LSB for Control 5',
    38: 'LSB for Control 6',
    39: 'LSB for Control 7',
    40: 'LSB for Control 8',
    41: 'LSB for Control 9',
    42: 'LSB for Control 10',
    43: 'LSB for Control 11',
    44: 'LSB for Control 12',
    45: 'LSB for Control 13',
    46: 'LSB for Control 14',
    47: 'LSB for Control 15',
    48: 'LSB for Control 16',
    49: 'LSB for Control 17',
    50: 'LSB for Control 18',
    51: 'LSB for Control 19',
    52: 'LSB for Control 20',
    53: 'LSB for Control 21',
    54: 'LSB for Control 22',
    55: 'LSB for Control 23',
    56: 'LSB for Control 24',
    57: 'LSB for Control 25',
    58: 'LSB for Control 26',
    59: 'LSB for Control 27',
    60: 'LSB for Control 28',
    61: 'LSB for Control 29',
    62: 'LSB for Control 30',
    63: 'LSB for Control 31',
    64: 'Damper Pedal on/off',
    65: 'Portamento On/Off',
    66: 'Sostenuto On/Off',
    67: 'Soft Pedal On/Off',
    68: 'Legato Footswitch',
    69: 'Hold 2	≤63 off, ≥64',
    70: 'Sound Controller 1 (default: Sound Variation)',
    71: 'Sound Controller 2 (default: Timbre/Harmonic Intens.)',
    72: 'Sound Controller 3 (default: Release Time)',
    73: 'Sound Controller 4 (default: Attack Time)',
    74: 'Sound Controller 5 (default: Brightness)',
    75: 'Sound Controller 6 (default: Decay Time - see MMA RP-021)',
    76: 'Sound Controller 7 (default: Vibrato Rate - see MMA RP-021)',
    77: 'Sound Controller 8 (default: Vibrato Depth - see MMA RP-021)',
    78: 'Sound Controller 9 (default: Vibrato Delay - see MMA RP-021)',
    79: 'Sound Controller 10 (default undefined - see MMA RP-021)',
    80: 'General Purpose Controller 5',
    81: 'General Purpose Controller 6',
    82: 'General Purpose Controller 7',
    83: 'General Purpose Controller 8',
    84: 'Portamento Control',
    85: 'Undefined',
    86: 'Undefined',
    87: 'Undefined',
    88: 'High Resolution Velocity Prefix',
    89: 'Undefined',
    90: 'Undefined',
    91: 'Effects 1 Depth',
    92: 'Effects 2 Depth (formerly Tremolo Depth)',
    93: 'Effects 3 Depth',
    94: 'Effects 4 Depth (formerly Celeste [Detune] Depth)',
    95: 'Effects 5 Depth (formerly Phaser Depth)',
    96: 'Data Increment (Data Entry +1) (see MMA RP-018)',
    97: 'Data Decrement (Data Entry -1) (see MMA RP-018)',
    98: 'Non-Registered Parameter Number (NRPN) - LSB',
    99: 'Non-Registered Parameter Number (NRPN) - MSB',
    100: 'Registered Parameter Number (RPN) - LSB*',
    101: 'Registered Parameter Number (RPN) - MSB*',
    102: 'Undefined',
    103: 'Undefined',
    104: 'Undefined',
    105: 'Undefined',
    106: 'Undefined',
    107: 'Undefined',
    108: 'Undefined',
    109: 'Undefined',
    110: 'Undefined',
    111: 'Undefined',
    112: 'Undefined',
    113: 'Undefined',
    114: 'Undefined',
    115: 'Undefined',
    116: 'Undefined',
    117: 'Undefined',
    118: 'Undefined',
    119: 'Undefined',
    120: '[Channel Mode Message] All Sound Off',
    121: '[Channel Mode Message] Reset All Controllers',
    122: '[Channel Mode Message] Local Control',
    123: '[Channel Mode Message] All Notes Off',
    124: '[Channel Mode Message] Omni Mode Off (+ all notes off)',
    125: '[Channel Mode Message] Omni Mode On (+ all notes off)',
    126: '[Channel Mode Message] Mono Mode On (+ poly off, + all notes off)',
    127: '[Channel Mode Message] Poly Mode On (+ mono off, +all notes off)',
}